body
{
  background-color: #999999;
  color: #000000; margin: 0; padding: 0;
  font-family: "Lucida Grande", "Lucida Sans Unicode", "Helvetica Neue", Helvetica, Arial, Verdana, sans-serif;
  font-size: 12pt;
  line-height: 1.5em;
}

* { margin: 0; }

#login
{
    width: 95%;
    margin-left: auto;
    margin-right: auto;
    border: 1px solid #999999;
    background: -webkit-gradient(linear, left top, left bottom, from(#fff), to(#24414e));
    background: -moz-linear-gradient(top,  #fff,  #24414e);
    background-color: #24414e;
    -moz-border-radius: 10px;
    -webkit-border-radius: 10px;
    border-radius: 10px;
}
#login span.logo
{
    display: inline-block;
    margin-right: 5px;
    padding: 2px 10px 2px 20px;
    border-right: 1px solid #999999;
    font-size: 20px;
    font-weight: bolder;
    text-shadow: #efefef 1px 1px 0px;
}
#login label { color: #ffffff; text-shadow: 1px 1px 0px rgba(175, 210, 220, 0.8); }
#login input
{
    padding: 5px;
    background-color: #fAfAfA;
    border: 1px inset #999999;
    outline: none;
    -moz-border-radius: 3px; -webkit-border-radius: 3px; border-radius: 3px;
    box-sizing: border-box;
    -moz-box-sizing: border-box;
    -webkit-box-sizing: border-box;
}
#login input#host { width: 200px; }
#login input#port { width: 75px; }
#login input#password { width: 100px; }
#login button
{
    padding: 5px 10px 5px 10px;
    margin-left: 5px;
    text-shadow: #efefef 1px 1px 0px;
    border: 1px outset #999999;
    cursor: pointer;
    -moz-border-radius: 5px; -webkit-border-radius: 5px; border-radius: 5px;
}
#login button:hover
{
    background-color: #666666;
    color: #ffffff;
}

#spice-area
{
    height: 100%;
    width: 95%;
    padding: 0;
    margin-left: auto;
    margin-right: auto;
    border: solid #222222 1px;
    -webkit-box-shadow: 5px 5px 5px rgba(0, 0, 0, 0.2);
    -moz-box-shadow: 5px 5px 5px rgba(0, 0, 0, 0.2);
    box-shadow: 5px 5px 5px rgba(0, 0, 0, 0.2);
    -moz-border-radius: 10px;
    -webkit-border-radius: 10px;
    border-radius: 10px;
}
.spice-screen
{
    min-height: 600px;
    height: 100%;
    margin: 10px;
    padding: 0;
    background-color: #333333;
}
.spice-message {
    width: 700px;
    height: 50px;
    overflow: auto;
    margin-top: 5px;
    margin-left: auto;
    margin-right: auto;
    padding: 10px;
    background-color: #efefef;
    border: solid #c3c3c3 2px;
    font-size: 8pt;
    line-height: 1.1em;
    font-family: 'Andale Mono', monospace;
    -moz-border-radius: 10px;
    -webkit-border-radius: 10px;
    border-radius: 10px;
    -webkit-box-shadow: 5px 5px 5px rgba(0, 0, 0, 0.2);
    -moz-box-shadow: 5px 5px 5px rgba(0, 0, 0, 0.2);
    box-shadow: 5px 5px 5px rgba(0, 0, 0, 0.2);
}
.spice-message p {
    margin-bottom: 0em;
    margin-top: 0em;
}
.spice-message-warning {
    color: orange;
}
.spice-message-error {
    color: red;
}
