<!DOCTYPE html>
<html{% if request.LANGUAGE_CODE %} lang="{{ request.LANGUAGE_CODE }}"{% endif %}>

{% load pipeline %}
{% load static %}

<head>
    <meta charset="utf-8">
    <!--[if IE]><meta http-equiv="X-UA-Compatible" content="IE=edge" /><![endif]-->
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <title>Socrates</title>
    {% stylesheet 'socrates_css' %}
    <link rel="stylesheet" type="text/css" href="{% static "css/bootstrap.min.css" %}" />
</head>

<body>
<div id="socrates_app" class="container"></div>
{% javascript "socrates_js" %}
</body>

</html>
